<?php
/*
*
* JetBackup @ package
* Created By Idan Ben-Ezra
*
* Copyrights @ JetApps
* https://www.jetapps.com
*
**/

class Plugin_Jetbackup5 extends Plugin_SiteworxFeature {

	protected $_sw_menu_text = 'JetBackup 5';
	protected $_sw_menu_url = '/siteworx/jetbackup5';
	protected $_sw_menu_parent_big = 'iw-menu-features-misc';
	protected $_sw_menu_id = 'menu-jetbackup5';
	protected $_sw_menu_description = 'JetBackup 5 Interworx integration';
	protected $_sw_account_group_name = 'jetbackup5';
	protected $_sw_account_group_label = 'JetBackup 5 Plugin Options';
	protected $_sw_account_option_name = 'jetbackup5';
	protected $_sw_account_option_label = 'Access to JetBackup 5';
	protected $_sw_user_perm_name = 'jetbackup5';
	protected $_sw_user_perm_label = 'JetBackup 5';
	
	/**
	 * @return string
	 */
	public function getCategory() { return Plugin_Category::BACKUP_RESTORE; }

	/**
	 * @return int
	 */
	public function getPriority() { return 40; }

	/**
	 * @param string $username
	 * @param string $template
	 */
	private function _runAdminBin($username) {

		$cmd  = Ini::get(Ini::IWORX_BIN, 'runasuser');
		$cmd .= " $username custom /usr/local/jetapps/usr/bin/jetbackup5/jetbackup_admin 2>&1";

		$data = [];
		$input = file_get_contents('php://input');
		if($input) $data = array_merge($data, json_decode($input, true));
		if(isset($_REQUEST)) $data = array_merge($data, $_REQUEST);
		putenv('POST=' . json_encode($data));
		putenv('REQUEST_URI=' . $_SERVER['REQUEST_URI']);
		putenv('LANGUAGE=en');

		$this->_passthruWithHeaders($cmd);
		exit;
	}

	/**
	 * 
	 */
	public function runEnduser() {
    	$this->_runAdminBin(IW::SW()->getUniqname());
	}

	/**
	 * 
	 */
	public function runAdmin() {
		$this->_runAdminBin('root');
	}

	/**
	 * @param string $cmd
	 */
	private function _passthruWithHeaders($cmd) {
		$descriptors = [ 
			0 => [ 'pipe', 'r' ], // stdin
		    1 => [ 'pipe', 'w' ], // stdout
		    2 => [ 'pipe', 'w' ], // stderr 
		]; 

		$process = proc_open($cmd, $descriptors, $pipes);
		
		fclose($pipes[0]);
		
		if (is_resource($process)) {
			while ($s = fgets($pipes[1])) {
				header($s);
				if ($s == "\n" || $s == "\r\n") {
					break;
				}
			}
			while ($s = fgets($pipes[1])) {
				echo $s;
			}
		}
	}
	
	public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params) {

		if (
			substr($action, -6) !== 'Commit'/* ||
			!$Ctrl->isViewSuccess()*/
		) return;

		$vars = ($params instanceof Form) ? $this->_getParams($params) : [];
		$username = null;
		
		switch($ctrl_act) {
			case 'Ctrl_Nodeworx_Packages:addCommit':
			case 'Ctrl_Nodeworx_Packages:deleteCommit':
			case 'Ctrl_Nodeworx_Packages:editCommit':
				$username = 'root';
			break;
		
			case 'Ctrl_Nodeworx_Siteworx:suspendCommit':
			case 'Ctrl_Nodeworx_Siteworx:unsuspendCommit':
				$SW = new SiteWorx($vars['domain_copy']);
				$username = $SW->getUniqname();
			break;
			
			case 'Ctrl_Nodeworx_Siteworx:editCommit':
			case 'Ctrl_Nodeworx_Siteworx:addCommit':
			case 'Ctrl_Nodeworx_Siteworx:deleteUniqnameCommit':		
				if(isset($vars['uniqname'])) $username = $vars['uniqname'];
				elseif(isset($vars['user'])) $username = $vars['user'];
			break;
		}

		if($username) {
			$cmd  = Ini::get(Ini::IWORX_BIN, 'runasuser');
			$cmd .= " root custom \"/usr/local/jetapps/usr/bin/jetbackup5/sync_accounts $username\" 2>&1";
			$this->_passthruWithHeaders($cmd);
		}
		
		//@file_put_contents('/idan/__' . preg_replace("/[^a-zA-Z0-9:_]/", "-", trim($ctrl_act)), "Vars: " . (sizeof($vars) ? print_r($vars, 1) : 'NONE') . "\n");
	}

	private function _getParams(Form $Form) {
		$out = [];
		foreach ($Form as $Group) {
			foreach ($Group as $Input) {
				/* @var $Input Input_Input */
				foreach ($Input->getAliases() as $alias) {
					$value = $Input->getCurrentValue();
					if (is_array($value)) $value = implode(',', $value);
					$out[$alias] = $value;
				}
			}
		}
		
		return $out;
	}
	
	/**
	 * @param IWorxMenuManager $MenuMan
	 */
	public function updateNodeworxMenu(IWorxMenuManager $MenuMan) {

		$MenuMan->addMenuItemAfter(
			'iw-menu-sw-import',
			'menu-jetbackup5',
			[
				'text'                      => 'JetBackup 5',
				'url'                       => '/nodeworx/jetbackup5',
				'parent'                    => 'iw-menu-sw',
				'class'                     => 'mdi-power-plug',
				'disabled_for_reseller'     => '1'
			]
		);
	}
}
